
BOOKS = 'rnngen/resources/datasets/alice.txt'
SIMPSON = 'rnngen/resources/datasets/simpson.txt'
# SIMPSON_SHORT = 'rnngen/resources/datasets/simpsonshort.txt'  defect, cannot be read by utf-8.

BOOKS_PROCESSED = 'rnngen/resources/processeddata/alice_processed.txt'
SIMPSON_PROCESSED = 'rnngen/resources/processeddata/simpson_processed.txt'
SIMPSON_SHORT_PROCESSED = 'rnngen/resources/processeddata/simpson_short_processed.txt'
