from distutils.core import setup

setup(
    name='RnnGen',
    version='1.0',
    packages=['rnngen'],
    license='Creative Commons Attribution-Noncommercial-Share Alike license',
    long_description='Recurrent Neural Network Generative Model with Word2Vec',
)
